// Licensed to the .NET Foundation under one or more agreements.
// The .NET Foundation licenses this file to you under the MIT license.
//

//
// ===========================================================================
// File: rpcndr.h
//
// ===========================================================================
// dummy rpcndr.h for PAL

#ifndef __RPCNDR_H__
#define __RPCNDR_H__

#ifndef __RPCNDR_H_VERSION__
#define __RPCNDR_H_VERSION__
#endif

#endif  // __RPCNDR_H__
