// Licensed to the .NET Foundation under one or more agreements.
// The .NET Foundation licenses this file to you under the MIT license.
//

//
// ===========================================================================
// File: oaidl.h
//
// ===========================================================================

#ifndef __OAIDL_H__
#define __OAIDL_H__

// Pointer to IErrorInfo is still used in non-Windows code
// Prevent accidentally using its member
struct IErrorInfo;

#endif //__OAIDL_H__
