// Licensed to the .NET Foundation under one or more agreements.
// The .NET Foundation licenses this file to you under the MIT license.
//

//
// dummy ole2.h for PAL

#include "objidl.h"
#include "servprov.h"
#include "oaidl.h"

