#include "ProfilerCommon.h"
#include "CoreDumpWriter.h"
#include "Events.h"
#include "GenHelpers.h"
#include "Handle.h"
#include "Logging.h"
#include "Monitor.h"
#include "Procdump.h"
#include "ProcDumpConfiguration.h"
#include "Process.h"
#include "DotnetHelpers.h"
#include "ProfilerHelpers.h"
#include "Restrack.h"
#include "ProcDumpVersion.h"


